#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <net/if.h>
#include <arpa/inet.h>

#ifndef TEST
#include "dnit.h"
#include <net/nit_if.h>
#endif

#include "ifrag.h"
#include "packet.h"

char Packet[PACKETSIZE];
char pfrag1[256], pfrag2[PACKETSIZE];

char *get_ip_pak();

#ifndef TEST
Nit *nitfd;
#endif

main(argc, argv)
int argc;
char **argv;
{
	struct timeval timeout;
	struct sockaddr_in src, dst;
	struct ip *ihdr;
	struct tcphdr *thdr;
	int mtu;
	u_long pktlen;
	char ether_dhost[6];
	char *p;

	timeout.tv_sec  = 1;
	timeout.tv_usec = 0;

	if(argc != 4) {
		fprintf(stderr, "usage: %s <src host> <dst host> <mtu>\n", argv[0]);
		exit(1);
	}
	
	if (resolve_host (argv[1], &src) < 0) {
		fprintf(stderr, "cannot resolve %s\n", argv[1]);
		exit(1);
	}
	
	if (resolve_host (argv[2], &dst) < 0) {
		fprintf(stderr, "cannot resolve %s\n", argv[2]);
		exit(1);
	}

	if (!(mtu = atoi(argv[3])) || (mtu & 7)) {
		fprintf(stderr, "mtu incorrect.\n");
		exit(1);
	}
	
#ifndef TEST
	if ((nitfd = NitOpen("le0", NIT_BUFFER, 0, timeout,
	    NI_PROMISC | NI_TIMESTAMP | NI_DROPS | NI_LEN)) == NULL)
		exit(1);
#endif

	ihdr = (struct ip *) Packet;
	
	while(1) {
		if((p=get_ip_pak(&timeout, &pktlen, ether_dhost)) == NULL)
			continue;
	
		/* check whether our packet is bigger than our buffer. */
		if(pktlen > PACKETSIZE) {
			fprintf(stderr, "warning: packet is too big. dropping.\n");
			continue;
		}
		
		/* Yuk! I hate alignments... */
		bcopy(p, Packet, (int) pktlen);

		/* bahh.. we only want TCP packets */
		if(ihdr->ip_p != IPPROTO_TCP)
			continue;

		/* those packets are not for us */
		if(bcmp(&src.sin_addr, &ihdr->ip_src, 4))
			continue;
		if(bcmp(&dst.sin_addr, &ihdr->ip_dst, 4))
			continue;
		thdr = (struct tcphdr *) (p + ihdr->ip_hl * 4);

		if(thdr->th_flags != TH_SYN)
			continue;
		
		if(ip_frag_twoway(Packet, pfrag1, pfrag2, mtu) < 0) {
			fprintf(stderr, "Error occured.\n");
			break;
		}
		send_pak(pfrag2, pktlen - mtu, ether_dhost);
		sleep(1);
		send_pak(pfrag1, mtu + ihdr->ip_hl * 4, ether_dhost);
	}
#ifndef TEST
	NitClose(nitfd);
#endif
	exit(1);
}
